local lgr = love.graphics
local win = love.window
local lfs = love.filesystem
local kbd = love.keyboard
local tmr = love.timer

smbBlockProjectile = {}

function smbBlockProjectile:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v or {})
	return _i
end
function smbBlockProjectile:setup(v)
	self.world = v.world
	self.facing = 1
	self.x = v.x or 2
	self.y = v.y or 2
	self.w = v.w or width
	self.h = v.h or height
	self.vx = v.vx or 100
	self.vy = v.vy or 0
	self.kind = v.kind or "block"
	self.map = v.map or game.map
	local g
	if self.kind == "block" then
		g = 350
	else
		g = 200
	end
	local collisionTypes = { ["block"] = true, ["bomb"] = true, ["veg"] = false }
	self.collides = collisionTypes[self.kind]

	if self.kind == "bomb" then
		self.timer = v.timer or 3
	end

	self.id = v.id or "projectile"..math.random(1,99999999)
	self.isDead = false
	self.gravityAccel  = g -- pixels per second^2
	self.deadDuration  = 3   -- seconds until res-pawn
	self.runAccel      = 300 -- the player acceleration while going left/right
	self.brakeAccel    = 500
	self.jumpVelocity  = 200 -- the initial upwards velocity when jumping

	self.beingThrown = v.beingThrown or false
	self.isProjectile = v.isProjectile or false

	self.timesBounced = 0
	self.maxBounceCount = v.maxBounce or 3
end

function smbBlockProjectile:update(dt)
	self:changeVelocityByGravity(dt)
	self.vy = math.max(math.min(self.vy, 250), -250)
	self.vx = math.max(math.min(self.vx, 100), -100)

	if self.timer then
		self.timer = self.timer - dt
		if self.timer <= 0 then
			self.timer = 0
			self.isDead = true
			print("DEAD")
			game:createExplosion(self.x + self.w / 2, self.y + self.h / 2)
		end
	end

	if self.collides then
		self:moveColliding(dt)
		self:changeVelocityByBeingOnGround(dt)
	else
		self:moveCollidingPassthrough(dt)
	end

	if self.isDead then
		if self.kind == "block" then
			local x = self.tx
			local y = self.ty
			self.map.grid[x+1][y+1] = 3

			local n = "block" .. math.random(0,1000000)
			self.map.throwableBlockList[n] = {
				x = x * self.map.tileSize, y = y * self.map.tileSize, w = 1 * self.map.tileSize, h = 1 * self.map.tileSize, wallType = 3, kind = "block", canPickUp = true, id = n }

			self.map.collisionMap:add(
				self.map.throwableBlockList[n], self.map.throwableBlockList[n].x, self.map.throwableBlockList[n].y, self.map.throwableBlockList[n].w, self.map.throwableBlockList[n].h)
		else

		end
	end
end

function smbBlockProjectile:draw()
	lgr.push()
	lgr.translate(math.ceil(self.x), math.ceil(self.y))
	if self.kind == "block" then
		lgr.push()
		lgr.translate(-2, -8)
		lgr.setColor(100,200,255)
		lgr.rectangle("fill", 0, 0, 16, 16 / 2)
		lgr.setColor(0,255,255)
		lgr.rectangle("fill", 0 + 2, 0 + 16 / 2, 16 - 4, 16 / 2)
		lgr.pop()
	else
		lgr.setColor(100,200,255)
		lgr.rectangle("fill", 0, 0, self.w, self.h)
	end
	lgr.setColor(255,0,0,200)
	if self.timer then
		if self.timer <= 1 then
			if math.cos(self.timer * 50) >= 0 then
				lgr.setColor(255,255,255,200)
			end
		end
	end
	lgr.rectangle("fill", 0, 0, self.w, self.h)
	if self.timer then
		lgr.setFont(font.tiny)
		lgr.setColor(0,0,0)
		lgr.printf(math.ceil(self.timer), 0, 1, 8, "center")
	end
	lgr.setFont(font.tiny)
	lgr.setColor(255,255,255)
	lgr.print(tostring(self.beingThrown), 0, 0)

	lgr.pop()
end

function smbBlockProjectile:moveCollidingPassthrough(dt)
	self.onGround = false
	local world = self.world

	local future_x = self.x + self.vx * dt
	local future_y = self.y + self.vy * dt

	local next_x, next_y, cols, len = world:move(self, future_x, future_y, function(other) return false end)

	self.x, self.y = next_x, next_y

	if self.y > 500 then
		print("DEAD")
		self.isDead = true
	end
end

function smbBlockProjectile:changeVelocityByBeingOnGround(dt)
	if self.isDead then return end
	if self.onGround then
		self.vy = math.min(self.vy, 0)
		if self.vx > 0 then
			self.vx = self.vx - dt * 300
			if self.vx < 0 then self.vx = 0 end
		elseif self.vx < 0 then
			self.vx = self.vx + dt * 300
			if self.vx > 0 then self.vx = 0 end
		end

		-- print(self.vx, self.vy)
		if self.vx == 0 and self.vy == 0 then
			if self.kind == "block" then
				self.x = math.floor((self.x / 16) + 0.5) * 16
				self.isDead = true
				print("Landed")
			elseif self.kind == "bomb" then

			end

			self.tx = math.floor(self.x / 16)
			self.ty = math.floor(self.y / 16)
		end
	end
end

function smbBlockProjectile:checkIfOnGround(ny)
	self.isJumping = false
	if ny < 0 then self.onGround = true end
end

local blockProjectileFilter = function(other)
	if other.kind == "player" or other.kind == "bomb" or other.kind == "enemy" then
		return false
	end
	if other.slippery then
		return "slide"
	end
	return "bounce"
end

function smbBlockProjectile:moveColliding(dt)
	self.onGround = false
	local world = self.world

	local future_x = self.x + self.vx * dt
	local future_y = self.y + self.vy * dt

	local next_x, next_y, cols, len = world:move(self, future_x, future_y, blockProjectileFilter)

	for i=1, len do
		local col = cols[i]
		local other = col.other

		if col.normal.y == -1 then
			self.onIce = other.slippery or false

			if other.isPlatform then
				self.attachedToPlatform = other.nameID
			end

			if other.conveyorDirection then
				next_x = next_x + dt * (16 * other.conveyorDirection)
			end
		end

		self:changeVelocityByCollisionNormal(col.normal.x, col.normal.y, bounciness)
		self:checkIfOnGround(col.normal.y)
	end

	self.x, self.y = next_x, next_y

	if self.y > 500 then
		print("DEAD")
		self.isDead = true
	end
end

function smbBlockProjectile:changeVelocityByGravity(dt)
	self.vy = self.vy + self.gravityAccel * dt
end

function smbBlockProjectile:changeVelocityByCollisionNormal(nx, ny, bounciness)
	bounciness = bounciness or 0.3
	local vx, vy = self.vx, self.vy

	if (nx < 0 and vx > 0) or (nx > 0 and vx < 0) then
		vx = -vx * bounciness
	end

	if (ny < 0 and vy > 0) or (ny > 0 and vy < 0) then
		vy = -vy * bounciness
		self.timesBounced = self.timesBounced + 1
		if self.timesBounced > self.maxBounceCount and vy < 0.01 then vy = 0 end
	end

	self.vx, self.vy = vx, vy
end

function smbBlockProjectile:move(l,t)
  self.x, self.y = l,t
  self.world:move(self, l, t)
end

function smbBlockProjectile:getCenter()
  return self.x + self.w / 2,
         self.y + self.h / 2
end

function smbBlockProjectile:destroy()
  self.world:remove(self)
end
