local lgr = love.graphics
local win = love.window
local lfs = love.filesystem
local kbd = love.keyboard
local tmr = love.timer

shyguy = {}

function shyguy:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v or {})
	return _i
end
function shyguy:setup(v)
	self.world = v.world
	self.facing = 1
	self.x = v.x or 2 * 16
	self.y = v.y or 2 * 16
	self.w = v.w or 12
	self.h = v.h or 12
	self.vx = 0
	self.vy = 0
	self.iox = v.iox or 0
	self.ioy = v.ioy or 0
	self.isDead = false
	self.map = v.map or game.map
	self.gravityAccel  = 600 -- pixels per second^2
	self.deadDuration  = 3   -- seconds until res-pawn
	self.runAccel      = 300 -- the player acceleration while going left/right
	self.brakeAccel    = 500
	self.iceAccel	   = 100
	self.iceBrake	   = 160
	self.waterAccel	   = 40
	self.waterBrake	   = 100
	self.runSpeed      = 16
	self.jumpVelocity  = 300 -- the initial upwards velocity when jumping
	self.jumpHeightMax = 48
	self.jumpHeight = 0
	self.isPlatform = true
	self.solidTop = true
	self.ladderClimbSpeed = 64
	self.jumpCooldown = 0
	self.kind = "enemy"
	self.nameID = v.nameID
	self.pState = v.pState
	self.attachedToPlatform = ""

	self.leftWater = true
	self._wasInWater = false

	self.pausingForAnimation = ""
	self.pauseTime = 0
	self.frame = 0

	self.moving = -1
end

function shyguy:update(dt)
	if self.pauseTime == 0 then
		self:changeVelocityByAI(dt)
		self:changeVelocityByGravity(dt)
	end

	local _runSpeed = self.runSpeed
	if self.inWater then _runSpeed = 70 end

	self.vy = math.max(math.min(self.vy, 255), -255)
	self.vx = math.max(math.min(self.vx, _runSpeed), -_runSpeed)

	self:moveColliding(dt)
	self:changeVelocityByBeingOnGround(dt)

	if self.y > (self.map.height + 2) * self.map.tileSize then
		self.dead = true
	end
end

function shyguy:draw()
	lgr.setColor(255,0,0)
	lgr.rectangle("fill", self.x, self.y, self.w, self.h)
end

function shyguy:jump(w, m)
	if self.onGround or self.onLadder or self.inWater or w then
		local jumpVel
		if self.inWater then
			jumpVel = self.jumpVelocity / 5
		else
			jumpVel = self.jumpVelocity
		end
		self.onLadder = false
		self.vy = -jumpVel
		self.isJumping = true
		self.jumpHeight = 0
		self.jumpCooldown = 0.1
		self.jumpNow = false
	end
end

function shyguy:changeVelocityByBeingOnGround(dt)
	if self.onGround then
		self.vy = math.min(self.vy, 0)
	end
end

function shyguy:checkIfOnGround(ny)
	self.isJumping = false
	if ny < 0 then
		self.onGround = true
	end
end

local shyguyFilter = function(other)
	local p = game.player
	local okind = other.kind

	if okind == "player" then return false end
	if okind == "enemy" then return false end
	if okind == "bomb" then return false end
	if other.isProjectile then
		if (okind == "block" and not other.beingThrown) or not okind == "bomb" then return "slide" end
		return "cross"
	end
	if not (other.solidTop and p.y + p.h > other.y) then return "slide" end
	if other.solid then return "slide" end
	return false
end

function shyguy:moveColliding(dt)
	self.onGround = false
	local world = self.world

	local xs, ys = self.vx, self.vy

	local future_x = self.x + xs * dt
	local future_y = self.y + ys * dt

	local next_x, next_y, cols, len = world:move(self, future_x, future_y, shyguyFilter)

	self.attachedToPlatform = ""
	for i=1, len do
		local col = cols[i]
		local other = col.other

		if col.normal.x == -self.moving then
			self.moving = -self.moving
		end

		if col.normal.y == -1 then
			self.onIce = other.slippery or false

			if other.isPlatform then
				self.attachedToPlatform = other.nameID

				if not other.active and other.activateOnTouch then
					other.active = true
				end
			end

			if other.conveyorDirection then
				next_x = next_x + dt * (16 * other.conveyorDirection)
			end
			if self.standingOnBlock == 3 then
				self.standingOnBlockID = other.id or 0
			end
			self.pickBlockX = math.ceil((self.x + (self.w / 2)) / game.map.tileSize)
			self.pickBlockY = math.ceil((self.y + self.h) / game.map.tileSize) + 1
		end

		self:changeVelocityByCollisionNormal(col.normal.x, col.normal.y, bounciness)
		self:checkIfOnGround(col.normal.y)
	end

	self.x, self.y = next_x, next_y

	if game.player.attachedToPlatform == self.nameID then
		game.player:moveWithPlatform(dt, xs, ys)
	end
end

function shyguy:moveWithPlatform(dt, vx, vy)
	local world = self.world

	local future_x = self.x + vx * dt
	local future_y = self.y + vy * dt

	local next_x, next_y, cols, len = world:move(self, future_x, future_y, shyguyFilter)

	self.x, self.y = next_x, next_y
end

function shyguy:changeVelocityByAI(dt)
	if self.isDead then return end

	local vx, vy = self.vx, self.vy
	local brakeAccel, runAccel

	if self.inWater then
		brakeAccel = self.brakeAccel
		runAccel = self.runAccel
	elseif self.onIce then
		brakeAccel = self.iceBrake
		runAccel = self.iceAccel
	elseif self.onGround then
		brakeAccel = self.brakeAccel
		runAccel = self.runAccel
	else
		brakeAccel = self.brakeAccel * 0.6
		runAccel = self.runAccel * 0.6
	end

	if self.moving < 0 then
		vx = vx - dt * (vx > 0 and brakeAccel or runAccel)
		self.facing = -1
	elseif self.moving > 0 then
		vx = vx + dt * (vx < 0 and brakeAccel or runAccel)
		self.facing = 1
	else
		local brake = dt * (vx < 0 and brakeAccel or -runAccel)
		if math.abs(brake) > math.abs(vx) then
			vx = 0
		else
			vx = vx + brake
		end
	end

	self.vx, self.vy = vx, vy
end

function shyguy:changeVelocityByGravity(dt)
	local grav = self.gravityAccel
	if self.inWater then grav = 50 end
	self.vy = self.vy + grav * dt
end

function shyguy:changeVelocityByCollisionNormal(nx, ny, bounciness)
	if self.onLadder then return end
	bounciness = bounciness or 0
	local vx, vy = self.vx, self.vy

	if (nx < 0 and vx > 0) or (nx > 0 and vx < 0) then
		vx = -vx * bounciness
	end

	if (ny < 0 and vy > 0) or (ny > 0 and vy < 0) then
		vy = -vy * bounciness
	end

	self.vx, self.vy = vx, vy
end

function shyguy:move(l,t)
  self.x, self.y = l,t
  self.world:move(self, l, t)
end

function shyguy:getCenter()
  return self.x + self.w / 2,
         self.y + self.h / 2
end

function shyguy:destroy()
  self.world:remove(self)
end
