local lfs = love.filesystem
local lgr = love.graphics
local win = love.window
local tmr = love.timer

function exportGlobalLog()
	dt = nil
	local transferTable = {}
	local i = 1
	for k, v in pairs(_G) do
		transferTable[i] = { name = k, type = type(v), value = tostring(v) }
		i = i + 1
	end

	table.sort( transferTable,
		function(a, b)
			return a.type > b.type
		end
	)

	local t = "GLOBAL EXPORT LOG:\n"
	for k, v in pairs(transferTable) do
		if not string.find(v.value, "builtin") then
			t = t .. "  " .. v.type .. "  " .. v.name .. "  " .. v.value .. "\n"
		end
	end
	lfs.write("global_export.log", t)
end

function overlap(x1,y1,w1,h1, x2,y2,w2,h2)
  return not (x1+w1 < x2  or x2+w2 < x1 or y1+h1 < y2 or y2+h2 < y1)
end

function math.distanceFrom(x1,y1,x2,y2) return math.sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2) or 0 end

function math.findAngle(x1, y1, x2, y2, rad)
	if rad then
		return math.atan2(y1 - y2, x2 - x1)
	else
		return math.deg(math.atan2(y1 - y2, x2 - x1))
	end
end

function string.createUUID(n)
    local uuid = ""
    local n = n or 25
    local chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
    for i = 1, n do
        local l = math.random(1, #chars)
        uuid = uuid .. string.sub(chars, l, l)
    end
    --print("Generated UUID:", uuid)
    return uuid
end

function copyGrid(oldGrid)
	local newGrid = {}
	for x, gg in pairs(oldGrid) do
		newGrid[x] = {}
		for y, gg in pairs(oldGrid[x]) do
			newGrid[x][y] = oldGrid[x][y]
		end
	end
	return newGrid
end

function math.round(val, decimal)
	if not val then return 0 end
	if (decimal) then
		return math.floor( (val * 10^decimal) + 0.5) / (10^decimal)
	else
		return math.floor(val+0.5)
	end
end

function lgr.prints(t, x, y, ...)
	local r, g, b, a = lgr.getColor()
	lgr.setColor(0,0,0,a)
	lgr.print(t, x + 1, y + 1, ...)
	lgr.setColor(r,g,b,a)
	lgr.print(t, x, y, ...)
end

function resizeScreen(w, h, highdpi)
	local w = w or lgr.getWidth()
	local h = h or lgr.getHeight()
	local s = 2
	if highdpi then dpis = 2 end
	screen = {
		drawWidth = w, --960
		drawHeight = h, --540
		pixelScale = love.window.getPixelScale(),
		simpleMode = false,
		overallScale = s,
		debugTextScale = dpis or 1,
		focused = false
	}
	screen.zoomScale = 4.5 * (screen.drawWidth / 1920)
end

function math.clamp(n, low, high)
	return math.min(math.max(n, low), high)
end

function capitalize(str)
	return (str:gsub("^%l", string.upper))
end

function numberWord(num)
	local word = {"one", "two", "three", "four", "five", "six", "seven", "eight", "nine", "ten", "eleven", "twelve", "thirteen", "fourteen", "fifteen", "sixteen", "seventeen", "eighteen", "ninteen", "twenty"}
	return word[num] or num
end

function drawCursor(x, y, ss)
	local spinSpeed = ss or 5
	lgr.push()
	lgr.translate(x, y)

	lgr.push()
	lgr.translate(0,1)
	lgr.setColor(200,200,0)
	lgr.line(-16.5, -0.5, -1.5, -0.5)
	lgr.polygon("fill", 0, 0, -15, 8 * math.cos(tmr.getTime() * spinSpeed), -17, 0, -15, -8 * math.cos(tmr.getTime() * spinSpeed))
	lgr.pop()
	lgr.setColor(255,255,0)
	lgr.line(-16.5, -0.5, -1.5, -0.5)
	lgr.polygon("fill", 0, 0, -15, 8 * math.cos(tmr.getTime() * spinSpeed), -17, 0, -15, -8 * math.cos(tmr.getTime() * spinSpeed))

	lgr.pop()
end

function formatTime(t)
	return os.date("!%X",t)
	-- local r = math.floor(math.min(t/60/60,60)) .. ":" .. string.sub("0" .. math.floor(math.min(t/60,60)),-2) .. ":" .. string.sub("0" .. math.floor(math.min(t,60)),-2)
	-- if m then r = r .. (t - math.floor(t)) end
	-- return r
end

function drawSpinningMessageCursor(x, y)
	-- local o = ((math.pi / 2) + math.cos(tmr.getTime() * 2)) / math.pi
	-- print(o)
	lgr.setBlendMode("additive")

	lgr.push()
	lgr.translate(x, y)
	lgr.rotate(tmr.getTime() * 6)
	lgr.setColor(0,0,255)
	lgr.rectangle("fill", -5, -5, 10, 10)
	-- lgr.rectangle("line", -4.5, -4.5, 9, 9)
	-- lgr.rectangle("fill", -(5 - o * 4), -(5 - o * 4), 10 - o * 8, 10 - o * 8)
	lgr.pop()

	lgr.push()
	lgr.translate(x, y)
	lgr.rotate(tmr.getTime() * 2)
	lgr.setColor(0,255,0)
	lgr.rectangle("fill", -5, -5, 10, 10)
	-- lgr.rectangle("line", -4.5, -4.5, 9, 9)
	-- lgr.rectangle("fill", -(5 - o * 4), -(5 - o * 4), 10 - o * 8, 10 - o * 8)
	lgr.pop()

	lgr.push()
	lgr.translate(x, y)
	lgr.rotate(tmr.getTime() * 4)
	lgr.setColor(255,0,0)
	lgr.rectangle("fill", -5, -5, 10, 10)
	-- lgr.rectangle("line", -4.5, -4.5, 9, 9)
	-- lgr.rectangle("fill", -(5 - o * 4), -(5 - o * 4), 10 - o * 8, 10 - o * 8)
	lgr.pop()

	lgr.setBlendMode("alpha")
end
