local lgr = love.graphics
local win = love.window
local tmr = love.timer
local lms = love.mouse

local _stateStack = {}

function _stateStack:init(...)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(...)
	return _i
end
function _stateStack:setup(...)
	local arg = ... or {}
	self.stateStack = {}
	self.stateStackLevel = 0
	self.drawWidth = arg.drawWidth or lgr.getWidth()
	self.drawHeight = arg.drawHeight or lgr.getHeight()
	self.drawScale = 1
	self.useScissors = arg.useScissors or false

	self.name = arg.name or "Stack Unnamed"
	self.debuggerEnabled = false
	-- if arg.useCanvas then
	-- 	self.canvas = lgr.newCanvas(arg.canvasWidth, arg.canvasHeight)
	-- end
	self:resize()
	print("Loaded Statestack:", self.name)
end

function _stateStack:push(s, ...)
	if not s then return false end
	if s.loaded then return false end

	local n = #self.stateStack or 0
	if self.stateStack then
		if n > 0 then
			if self.stateStack[self.stateStackLevel].blur then
				self.stateStack[self.stateStackLevel]:blur()
			end
		end
	end
	self.stateStack[n + 1] = s
	self.stateStackLevel = #self.stateStack
	self.stateStack[self.stateStackLevel]:load(...)
	self.stateStack[self.stateStackLevel].id = #self.stateStack
	local w, h = self.drawWidth, self.drawHeight
	if self.stateStack[self.stateStackLevel].canvas then
		w = self.stateStack[self.stateStackLevel].canvas:getWidth()
		h = self.stateStack[self.stateStackLevel].canvas:getHeight()
	end
	if not self.stateStack[self.stateStackLevel].align then self.stateStack[self.stateStackLevel].align = {} end
	if not self.stateStack[self.stateStackLevel].align.h then self.stateStack[self.stateStackLevel].align.h = "center" end
	if not self.stateStack[self.stateStackLevel].align.v then self.stateStack[self.stateStackLevel].align.v = "center" end

	if not self.stateStack[self.stateStackLevel].width then self.stateStack[self.stateStackLevel].width = w end
	if not self.stateStack[self.stateStackLevel].height then self.stateStack[self.stateStackLevel].height = h end

	if self.stateStack[self.stateStackLevel].focus then self.stateStack[self.stateStackLevel]:focus() end

	print("Pushing state", s.name)

	collectgarbage()

	return true
end

function _stateStack:replace(s, ...)
	if not s then return false end
	if s.loaded then return false end

	self:release()
	self.stateStack = {}

	self:push(s, ...)
end

function _stateStack:release()
	for i = self.stateStackLevel, 1, -1 do
		if self.stateStack[i].blur then self.stateStack[i]:blur() end
		self.stateStack[i]:unload()
		self.stateStack[i] = nil
	end
end

function _stateStack:back(pass)
	self.stateStack[self.stateStackLevel]:blur()
	self.stateStack[self.stateStackLevel]:unload()
	self.stateStack[#self.stateStack] = nil
	self.stateStackLevel = #self.stateStack
	self.stateStack[self.stateStackLevel]:focus(pass)
end

function _stateStack:goto(level, ...)
	if not level then return false end
	if level >= self.stateStackLevel then return false end
	for i = self.stateStackLevel, level + 1, -1 do
		self.stateStack[i]:blur()
		self.stateStack[i]:unload()
		self.stateStack[i] = nil
	end

	self.stateStackLevel = #self.stateStack
	self.stateStack[self.stateStackLevel]:focus(...)
end

function _stateStack:update(dt)
	if not self.stateStack[self.stateStackLevel] then return false end
	for i = 1, self.stateStackLevel do
		if self.stateStack[i].update then
			if i == #self.stateStack or self.stateStack[i].backgroundUpdate then
				self.stateStack[i]:update(dt)
			end
		end
	end
	-- local t = (self.stateStack[self.stateStackLevel].title or "NO TITLE") .. " - FPS: " .. tmr.getFPS()
	-- win.setTitle(t)
end

function _stateStack:draw()
	lgr.origin()

	local DRAWSCALEVERT = 1
	if not self.stateStack[self.stateStackLevel] then return false end

	for i = 1, self.stateStackLevel do
		if self.stateStack[i].canvas then

			-- lgr.setCanvas(self.stateStack[i].canvas)
			-- self.stateStack[i].canvas:clear(100,0,0)

			self.stateStack[i]:draw()

			-- lgr.setCanvas()
		end

		if self.useScissors then
			lgr.setScissor(self.horizOffset, self.vertOffset, self.drawScale * self.drawWidth, self.drawScale * self.drawHeight)
		end

		lgr.push()
		lgr.translate(self.horizOffset, self.vertOffset)
		if not self.stateStack[i].ignoreTransformations then
			lgr.scale(self.drawScale)
		end

		local o = 255 * (self.stateStack[i].opacity or 1)
		local r = self.stateStack[i].rotation or 0
		local s = self.stateStack[i].scale or 1

		local blackOut = false
		if i == #self.stateStack - 1 and self.stateStack[i+1].forceBlackout then
			blackOut = true
		end
		if (i == #self.stateStack or self.stateStack[i].backgroundDraw) and not blackOut then
			if not self.stateStack[i].align then self.stateStack[i].align = {} end
			local ah = self.stateStack[i].align.h or "center"
			local av = self.stateStack[i].align.v or "center"

			local h1, v1 = self.drawWidth / 2, self.drawHeight / 2
			if ah == "left" then h1 = 0 elseif ah == "right" then h1 = -lgr.getWidth() end
			if av == "top" then v1 = 0 elseif av == "bottom" then v1 = lgr.getHeight() end

			local h2, v2 = -(self.stateStack[i].width / 2), -(self.stateStack[i].height / 2)
			if ah == "left" then h2 = 0 elseif ah == "right" then h2 = self.stateStack[i].width end
			if av == "top" then v2 = 0 elseif av == "bottom" then v2 = -self.stateStack[i].height end

			if not self.stateStack[i].canvas then
				self.stateStack[i]:draw()
			else
				local sc = self.stateStack[i].canvas

				if self.stateStack[i].drawScale then
					s = self.stateStack[i].drawScale
				end

				local ss = screen.drawHeight / sc:getHeight()
				lgr.setColor(255,255,255,o)
				lgr.draw(sc, (self.stateStack[i].horizOffset or 0), (self.stateStack[i].vertOffset or 0), 0, ss or 1)
			end
		end


		lgr.pop()

		if self.useScissors then
			lgr.setScissor()
		end
	end

	if self.debuggerEnabled then
		lgr.push()
		lgr.translate(lgr.getWidth() - 200, 300)
		lgr.setFont(font.cga)
		lgr.setColor(255,255,255)
		lgr.print("State List:", 0, 0)
		for i = 1, self.stateStackLevel do
			lgr.print(i .. ") " .. (self.stateStack[i].name or "<<NIL>>") .. " " .. tostring(self.stateStack[i].backgroundDraw), 0, i * 10)
		end

		lgr.setColor(255,255,255)
		lgr.print(self.name, 0, 100)

		lgr.pop()
	end
end

function _stateStack:resize(w, h)
	local w = w or lgr.getWidth()
	local h = h or lgr.getHeight()
	self.drawScale = screen.drawScale or 1

	self.horizOffset = (w - (self.drawScale * self.drawWidth)) / 2
	self.vertOffset = (h - (self.drawScale * self.drawHeight)) / 2

	self.fullWidth = w
	self.fullHeight = h

	if not self.stateStack[self.stateStackLevel] then return false end
	for i = 1, self.stateStackLevel do
		if self.stateStack[i].resize then
			if i == #self.stateStack or self.stateStack[i].backgroundUpdate then
				self.stateStack[i]:resize(w, h)
			end
		end
	end
end

function _stateStack:keypressed(k, u)
	if not self.stateStack[self.stateStackLevel] then return false end
	for i = 1, self.stateStackLevel do
		if self.stateStack[i].keypressed then
			if i == #self.stateStack or self.stateStack[i].backgroundKeyEvents then
				self.stateStack[i]:keypressed(k, u)
			end
		end
	end
end

function _stateStack:keyreleased(k, u)
	if not self.stateStack[self.stateStackLevel] then return false end
	for i = 1, self.stateStackLevel do
		if self.stateStack[i].keyreleased then
			if i == #self.stateStack or self.stateStack[i].backgroundKeyEvents then
				self.stateStack[i]:keyreleased(k, u)
			end
		end
	end
end

function _stateStack:mousepressed(x, y, button)
	if not self.stateStack[self.stateStackLevel] then return false end
	for i = 1, self.stateStackLevel do
		if i == #self.stateStack and self.stateStack[i].mousepressed then
			self.stateStack[i]:mousepressed(x, y, button)
		end
	end
end

function _stateStack:mousereleased(x, y, button)
	if not self.stateStack[self.stateStackLevel] then return false end
	for i = 1, self.stateStackLevel do
		if i == #self.stateStack and self.stateStack[i].mousereleased then
			self.stateStack[i]:mousereleased(x, y, button)
		end
	end
end

function _stateStack:gamepadPressed(joystick, button)
	print("Pressed:", joystick, button)
	if not self.stateStack[self.stateStackLevel] then return false end
	for i = 1, self.stateStackLevel do
		if self.stateStack[i].gamepadpressed then
			self.stateStack[i]:gamepadpressed(joystick, button)
		end
	end
end

function _stateStack:gamepadReleased(joystick, button)
	print("Released:", joystick, button)
	if not self.stateStack[self.stateStackLevel] then return false end
	for i = 1, self.stateStackLevel do
		if self.stateStack[i].gamepadreleased then
			self.stateStack[i]:gamepadreleased(joystick, button)
		end
	end
end

function _stateStack:getContext()
	return self.stateStack[self.stateStackLevel]
end

function _stateStack:loadState(file, arg)
	local arg = arg or {}
	if not file then return false end

-- function particle:create(v)
-- 	local _i = {}
-- 	setmetatable(_i, {__index = self})
-- 	_i:setup(v)
-- 	return _i
-- end

end

return _stateStack
