local lgr = love.graphics
local win = love.window
local lms = love.mouse
local kbd = love.keyboard
local tmr = love.timer
local lfs = love.filesystem

math.randomseed(os.time())
math.random() math.random() math.random()

flux = require "library.flux-master.flux"
cron = require "library.cron.cron"
easing = require "library.easing"
transition = require "library.transition"
stack = require "library.statestack"
trail = require 'library.trail'
richtext = require "library.richtext"
performanceGraph = require 'library.performancegraph'
bump = require 'library.bump.bump'

require "library.TEsound"
-- require "library.objectDraw"
require "library.dumper"
require "library.funct"
-- require "library.threeDeeWorld"

require "states.setupScreen"
require "states.smb2"

require "entities.player"
require "entities.projectile"
require "entities.particle"
require "entities.disposable"
require "entities.platform"
require "entities.shyguy"

local SHOWFPS = true
TILESIZE = 128
CHARACTERSCALE = 1.35 --2.65 previously

function love.load()
	print("---- RENDERER  ---- ")
    local name, version, vendor, device = love.graphics.getRendererInfo()
    print(string.format("Name: %s \nVersion: %s \nVendor: %s \nDevice: %s", name, version, vendor, device) .. "\n")
	-- lgr.setLineJoin("none")

	-- resizeScreen(853, 480)
	resizeScreen(1280, 720)
	-- resizeScreen(1280*2, 720*2, true)
	-- resizeScreen(1920, 1080)
	-- resizeScreen(1920*2, 1080*2, true)
	gameShader = love.graphics.newShader("library/blcknwht.frag")
	fogShader = love.graphics.newShader("library/fogshader.frag")

	lgr.setDefaultFilter("nearest", "nearest")
	font = {
		cga = lgr.newImageFont("fonts/cga.png", " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"),
		dialog = lgr.newImageFont("fonts/dialog.png", " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,!?-+/():;%&`_*#=[]'{}|<>"),
		dialogbold = lgr.newImageFont("fonts/dialogbold.png", " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,!?-+/():;%&`_*#=[]'{}|<>"),
		large = lgr.newImageFont("fonts/large.png", " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,!?-+/():;%&`_*#=[]'{}|<>"),
		mono2 = lgr.newImageFont("fonts/mono2.png", " ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789.,\"!?()_/-+:;\\<>"),
		tiny = lgr.newImageFont("fonts/tiny.png", " 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz.-,!:()[]{}<>\"'/=_"),
		symbol = lgr.newImageFont("fonts/symbol.png", "ç"),
		debug = lgr.newImageFont("fonts/dialog.png", " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,!?-+/():;%&`_*#=[]'{}|<>"),
	}
	font.cga:setLineHeight(1.1)
	font.dialog:setLineHeight(1.6)
	font.dialogbold:setLineHeight(1.6)
	font.debug:setLineHeight(1.1)
	font.tiny:setLineHeight(0.75)

	stateStack = stack:init { drawWidth = screen.drawWidth, drawHeight = screen.drawHeight, name = "Stack 1"}

	transition:init()

	love.resize()

	lgr.setLineWidth(1)
	lgr.setLineStyle("rough")
	lgr.setBackgroundColor(0,0,0)

	stateStack:push(setupScreen)

	kbd.setKeyRepeat(false)
end

function love.update(dt)
	dt = math.min(1/15, dt)
	screen.overlay:clear(0,0,0,0)

	performanceGraph:preUpdate()

	if screen.focused then
		TEsound.cleanup()
		flux.update(dt)
		stateStack:update(dt)
		cron.update(dt)
		transition:update(dt)
	end

	performanceGraph:postUpdate()
end

function love.draw()
	performanceGraph:preDraw()

	stateStack:draw()

	lgr.push()
	lgr.origin()
	lgr.setColor(255,255,255)
	lgr.draw(screen.overlay, stateStack.horizOffset, stateStack.vertOffset, 0, screen.pixelScale * screen.overlayPixelSize, screen.pixelScale * screen.overlayPixelSize)
	lgr.pop()

	transition:draw()

	if not screen.focused then
		lgr.setColor(0,0,0,100)
		lgr.rectangle("fill", 0, 0, win.getWidth(), win.getHeight())
	end

	performanceGraph:postDraw()

	if SHOWFPS then
		lgr.push()
		lgr.translate(lgr.getWidth() - 200 * screen.pixelScale, lgr.getHeight() - 8 * screen.pixelScale)

		lgr.push()
		lgr.scale(1 * screen.pixelScale)

		local m = math.floor(collectgarbage("count")/1024)
		lgr.setColor(0,255,0)
		lgr.setFont(font.cga)
		lgr.printf("Mem:" .. m .. "MB / FPS:"..tmr.getFPS(), 0, 0, 200, "right")
		lgr.rectangle("fill", -m + 200, -3, m, 1)

		lgr.pop()

		lgr.pop()
	end
end

function love.keypressed(k, r)
	stateStack:keypressed(k, r)

	if k == "f1" then
		performanceGraph:toggle()
	elseif k == "f8" then
		SHOWFPS = not SHOWFPS
	end
end

function love.keyreleased(k, r)
	stateStack:keyreleased(k, r)
end

function love.mousepressed(x, y, button)
	stateStack:mousepressed(x, y, button)
end

function love.focus(f)
	screen.focused = f
	if f then print("Focused") else print("Unfocused") end
end

function love.quit()
	exportGlobalLog()
end

function love.resize(w, h)
	local w = w or lgr.getWidth()
	local h = h or lgr.getHeight()
	print("Resized window:", w / screen.pixelScale, h / screen.pixelScale)

	local sw, sh = w / screen.drawWidth, h / screen.drawHeight

	local aspect_game = w / h
	local aspect_window = screen.drawWidth / screen.drawHeight

	if aspect_window <= aspect_game then
		screen.drawScale = sh
	else
		screen.drawScale = sw
	end

	local ho = stateStack.horizOffset / screen.pixelScale
	local vo = stateStack.vertOffset / screen.pixelScale
	screen.overlayPixelSize = 2
	if win.getHeight() <= 500 then
		screen.overlayPixelSize = 1
	end
	screen.overlay = lgr.newCanvas((w / screen.pixelScale) / screen.overlayPixelSize - ho, (h / screen.pixelScale) / screen.overlayPixelSize - vo)

	stateStack:resize(w, h)
end
